#pragma strict

public enum GameState { Boot, Ready, Playing, Win, Lose }

var START_LEVEL		: int = 1;
var LEVEL_COUNT		: int = 3;
var SECONDS			: int = 60.0;
var SKIN			: GUISkin;

private var first 			: boolean = true;
private var titleSeconds 	: float = 5.0;
private var hitGoal			: float = 0.6;		// dinamico por level
private var matches			: int = 0;			// partidas jogadas

@System.NonSerialized var state			: GameState = GameState.Boot;
@System.NonSerialized var currentLevel	: int = 1;
@System.NonSerialized var timeStart		: float = 0.0;		// Time.time when level started
@System.NonSerialized var timeElapsed	: float = 0.0;		// Seconds into level
@System.NonSerialized var timeHitting	: float = 0.0;		// Seconds HITTING in level

@System.NonSerialized var ISON			: boolean = false;	// Should PRESS??
@System.NonSerialized var PRESSING		: boolean = false;	// Pressing SPACE
@System.NonSerialized var HITTING		: boolean = false;	// Pressing RIGHT!!!
@System.NonSerialized var PROG			: float = 0.0;		// Level Prog 0.0 .. 1.0
@System.NonSerialized var PROGHIT		: float = 0.0;		// Hitting prog 0.0 .. 1.0 (POINTS)
@System.NonSerialized var NOISE			: float = 0.0;		// Current NOISE

// Scripts
@System.NonSerialized var OBJ 			: scr_OBJ;
@System.NonSerialized var CAMERA		: Camera;
@System.NonSerialized var VIGNETTE		: Vignetting;
@System.NonSerialized var MOTIONBLUR	: MotionBlur;
@System.NonSerialized var TXT_OBJ		: GameObject;
@System.NonSerialized var TXT_MSG		: GUIText;
@System.NonSerialized var TXT_SCORE		: GUIText;
@System.NonSerialized var BAR_GOAL		: GUITexture;
@System.NonSerialized var BAR_PROG		: GUITexture;
@System.NonSerialized var BAR_PROGHIT	: GUITexture;


function Awake () {
	// Make the game run even when in background
	Application.runInBackground = true;
}

function Start () {
	CAMERA = GameObject.Find("CAMERA").GetComponent(Camera);
	VIGNETTE = GameObject.Find("CAMERA").GetComponent(Vignetting);
	MOTIONBLUR = GameObject.Find("CAMERA").GetComponent(MotionBlur);
	
	// GUI
	TXT_OBJ = LIB.Find("TXT_MSG");
	TXT_MSG = TXT_OBJ.GetComponent(GUIText);
	TXT_SCORE = LIB.Find("TXT_SCORE").GetComponent(GUIText);
	BAR_GOAL = LIB.Find("BAR_GOAL").GetComponent(GUITexture);
	BAR_PROG = LIB.Find("BAR_PROG").GetComponent(GUITexture);
	BAR_PROGHIT = LIB.Find("BAR_PROGHIT").GetComponent(GUITexture);
	BAR_GOAL.pixelInset.y = Screen.height * -0.5;
	BAR_GOAL.pixelInset.height = 60;
	BAR_PROG.pixelInset.y = Screen.height * -0.5;
	BAR_PROG.pixelInset.height = 60;
	BAR_PROGHIT.pixelInset.y = Screen.height * -0.5;
	BAR_PROGHIT.pixelInset.height = 60;
	
	// load first level
	LoadLevel( START_LEVEL );
	state = GameState.Boot;

	if ( Debug.isDebugBuild )
	{
		SECONDS = 10.0;
		//titleSeconds = 1.0;
		//hitGoal = 0.6;
	}
}

function LoadLevel ( l : int ) : boolean {
	if ( l > LEVEL_COUNT )
		l = 1;
	if ( GetObj(l) == null )
		return false;
	
	// Turn ON/OFF objects
	for (var n = 1 ; n <= 100 ; n++)
	{
		var o : GameObject = GetObj(n);
		if (o) o.SetActive( n == l );
	}

	// Start !
	var obj : GameObject = GetObj(l);
	OBJ = obj.GetComponent(scr_OBJ);
	currentLevel = l;
	timeElapsed = 0.0;
	hitGoal = LIB.LMap( l, 1, LEVEL_COUNT, 0.8, 0.9 );
	PROG = 0.0;
	PROGHIT = 0.0;
	BAR_GOAL.enabled = false;
	BAR_PROG.enabled = false;
	BAR_PROGHIT.enabled = false;
	state = GameState.Ready;
	return true;
}

function GoToLevel ( l : int ) {
	if ( LoadLevel(l) )
		StartGame();
}

function StartGame () {
	BAR_GOAL.enabled = true;
	BAR_PROG.enabled = true;
	BAR_PROGHIT.enabled = true;
	timeStart = Time.time;
	timeHitting = 0.0;
	matches++;
	state = GameState.Playing;
}

function GetObj ( l : int ) : GameObject {
	var objname = "OBJ" + ( l < 10 ? "0"+l : l );
	var o : GameObject = LIB.Find(objname);
	return o;
}

function Update () {

	//
	// KEYBOARD
	if ( Debug.isDebugBuild )
	{
		if ( Input.GetKeyDown(KeyCode.Alpha1) )
			GoToLevel(1);
		if ( Input.GetKeyDown(KeyCode.Alpha2) )
			GoToLevel(2);
		if ( Input.GetKeyDown(KeyCode.Alpha3) )
			GoToLevel(3);
		if ( Input.GetKeyDown(KeyCode.Alpha4) )
			GoToLevel(4);
		if ( Input.GetKeyDown(KeyCode.Alpha5) )
			GoToLevel(5);
		if ( Input.GetKeyDown(KeyCode.Alpha6) )
			GoToLevel(6);
		if ( Input.GetKeyDown(KeyCode.Alpha7) )
			GoToLevel(7);
		if ( Input.GetKeyDown(KeyCode.Alpha8) )
			GoToLevel(8);
		if ( Input.GetKeyDown(KeyCode.Alpha9) )
			GoToLevel(9);
		if ( Input.GetKeyDown(KeyCode.Alpha0) )
			GoToLevel(10);
	}

	//
	// IS PRESSING SPACE????
	PRESSING = Input.GetKey(KeyCode.Space);
	

	//
	// UPDATE SCENE
	switch (state) {
		case GameState.Boot:
			// fade out title / msg
			TXT_SCORE.text = "";
			TXT_MSG.material.color.a = LIB.LMapClamp( Time.time, titleSeconds*0.5, titleSeconds, 1.0, 0.0 );
			TXT_OBJ.SetActive( true );
			// Ready for game?
			//if ( Time.time > titleSeconds || Input.GetKeyDown(KeyCode.Space) )
			if ( Time.time > titleSeconds )
			{
#if UNITY_IPHONE
				TXT_MSG.text = "press to start";
#else
				TXT_MSG.text = "press space to start";
#endif
				state = GameState.Ready;
				timeStart = Time.time;
			}
			break;
		case GameState.Ready:
			TXT_SCORE.text = "";
			if (matches == 0)
				TXT_MSG.material.color.a = LIB.LMapClamp( Time.time, timeStart, timeStart+titleSeconds*0.5, 0.0, 1.0 );
			else
				TXT_MSG.material.color.a = LIB.LMapClamp( Time.time, timeStart+titleSeconds*0.5, timeStart+titleSeconds, 1.0, 0.0 );
			// Start game?
			if ( Input.GetKeyDown(KeyCode.Space) && Time.time > (timeStart + (matches==0?0:1)) )
				StartGame();
			break;
		case GameState.Playing:
			ISON = OBJ.IsON();
			HITTING = ( ISON == PRESSING );
			PROGHIT = timeHitting / SECONDS;
			TXT_MSG.text = (ISON ? "press on color" : "release on black");
			TXT_MSG.material.color.a = ( first ? 1.0 : 0.0 );
			TXT_SCORE.text = "" + parseInt(PROGHIT*100) + " / " + parseInt(PROG*100) + " = " +  parseInt(PROGHIT*100)+ " %";
			//Debug.Log("ISON "+ISON+" / PRESSING "+PRESSING+" / HITTING "+HITTING);
			//
			// bars
			BAR_GOAL.pixelInset.x = Screen.width * -0.5 + Screen.width * hitGoal;
			BAR_GOAL.pixelInset.width = 2;
			BAR_PROG.pixelInset.x = Screen.width * -0.5 + Screen.width * PROG;
			BAR_PROG.pixelInset.width = 2;
			BAR_PROGHIT.pixelInset.x = Screen.width * -0.5;
			BAR_PROGHIT.pixelInset.width = Screen.width * PROGHIT;
			//
			// end game?
			if ( Time.time > (timeStart + SECONDS ) )
			{
				state = ( PROGHIT >= hitGoal ? GameState.Win : GameState.Lose );
				TXT_MSG.text = ( state == GameState.Win ? (currentLevel==LEVEL_COUNT?"congratulations!!!":"safe!") : "failed!" );
				TXT_MSG.material.color.a = 1.0;
				if (state == GameState.Win)
					first = false;
				LoadLevel( state == GameState.Win ? currentLevel+1 : currentLevel );
				timeStart = Time.time;
			}
			break;
		default:
			TXT_SCORE.text = "";
			ISON = true;
			HITTING = true;
			// Start new game?
			if ( Input.GetKeyDown(KeyCode.Space) )
			{
				TXT_MSG.material.color.a = 0.0;
				StartGame();
			}
		break;
	}

	// CALCS
	if ( state >= GameState.Playing)
	{
		timeElapsed = (Time.time - timeStart);
		PROG = timeElapsed / SECONDS;
	}
	NOISE = ( HITTING ? 0 : PROG );
	timeHitting += ( HITTING ? Time.deltaTime : 0 );

	// 
	// APPLY FX
	CAMERA.backgroundColor = OBJ.getColorBack();
	VIGNETTE.chromaticAberration = 25.0 * NOISE;
	MOTIONBLUR.enabled = ! HITTING;
}





function OnGUI () {
	if ( ! Debug.isDebugBuild )
		return;
		
	/*
	var y = Screen.height - 20;
	//GUI.Label (Rect (10, y, 500, 20), "TIME " + timeElapsed + " of " + SECONDS  + " / " + parseInt(prog * 100)  + " % ");
	GUI.Label (Rect (10, y, 500, 20), "" + parseInt(PROGHIT*100) + " / " + parseInt(PROG*100) );
	y -= 20;
	GUI.Label (Rect (10, y, 500, 20), "KEY " + ( PRESSING ? "ON" : "OFF" ) );
	y -= 20;
	GUI.Label (Rect (10, y, 500, 20), ( ISON ? "ON" : "OFF" ) );
	*/

}

